// Copyright (c) 2016-2019 United States Government as represented by the
// Administrator of the National Aeronautics and Space Administration.
// All Rights Reserved.
#include <fstream>
#include <thread>

#include "Test.h"

using namespace ARGH::Test;


/** Controls what tests to run by changing what's returned by ok(testNum).
 * Unless modified by processing args, ok(t) return true for every test index. */
class TestRange {
public:
    int low;
    int high;
    int maxi; // updated as ok(i) is called

    TestRange(int hi) : low(1), high(hi), maxi(0) { }

    /** Returns true if the testNum should be tested */
    bool ok(int testNum) {
        if (maxi < testNum) {
            maxi = testNum;
        }
        return (low <= testNum && testNum <= high);
    }
    
    /**
     If the arg is a single number, just run that test by setting low and high to it.
     Returns true if no arg or arg is a good number, else non-zero to abort.
     Could accept a comma-separated string of numbers, using a bitset.
    */
    bool handleArgsElseError(int argc, char* argv[]) {
        if (argc <= 1) { // 1 is the minimum. argv[0] is the program name
            return true;
        }
        if (argc == 2 && argv[1] != 0) {
            int val = strtol(argv[1], nullptr, 10); // returns zero on error
            if (low <= val && val <= high) {
                low = high = val;
                return true;
            } else if (val == 0) {
                std::cerr << "Error in arg, bad number, expected 1.." << high << ", was: " << argv[0] << std::endl;
                return false;
            }
        } else if (argc > 1) {
            std::cerr << "Expected 0 or 1 arg, 1.." << high << ", #args=" << (argc-1) << ", last arg=" << argv[argc-1] << std::endl;
            return false;
        }
        return 1;
    }

    /** Fails if a test was added with a higher ok(index) without raising high's initialization value */
    bool checkFailed() {
        if (low != high && maxi != high) {  // someone added an index, but didn't increment the high value
            std::cerr << "ERROR: high was " << high << ", but max index was: " << maxi << std::endl;
            return true;
        }
        return false;
    }
};


// Give a single numeric argument to run a single test
int main(int argc, char* argv[]) {
    TestContext context;
    TestRange tsts(18);

    if (!tsts.handleArgsElseError(argc, argv)) {
        return 1;
    }

    /*
    if (tsts.ok(1)) BatteryResultTests::registerTests(context);
    if (tsts.ok(2)) SimplifiedCircuitBatteryModelTests::registerTests(context);
    if (tsts.ok(3)) ConfigMapTests::registerTests(context);
    if (tsts.ok(4)) MatrixTests::registerTests(context);
    if (tsts.ok(5)) MessageBusTests::registerTests(context);
    if (tsts.ok(6)) PowerConsumptionModelTests::RegisterTests(context);
    if (tsts.ok(7)) ProximityTests::RegisterTests(context);
    if (tsts.ok(8)) TrajectoryTests::RegisterTests(context);
    if (tsts.ok(9)) GeographyTests::RegisterTests(context);
    if (tsts.ok(10)) TrajectoryTests2::RegisterTests(context);
    if (tsts.ok(11)) GeometryTests::RegisterTests(context);
    if (tsts.ok(12)) StatePlaneParameterTests::RegisterTests(context);
    if (tsts.ok(13)) BigIntTests::RegisterTests(context);
    if (tsts.ok(14)) math_tests::RegisterTests(context);
    if (tsts.ok(15)) pcm_integration_tests::RegisterTests(context);
    if (tsts.ok(16)) geo::angle_tests::RegisterTests(context);
    if (tsts.ok(17)) simple_trajectory_predictor_tests::RegisterTests(context);
    if (tsts.ok(18)) DynamicDensityTests::registerTests(context);
    if (tsts.checkFailed()) {
        return 1;
    }
    
    usize result = context.Execute();
    std::ofstream junit("testresults/sws-metrics.xml");
    context.WriteJUnit(junit);
    return static_cast<int>(result);
    */
}
